@@ -108,8 +108,8 @@
 #' Hardy, G., Littlewood, J. E., and Polya, G. (1952). *Inequalities* (2nd
 #' edition). Cambridge University Press.
 #'
-#' ILO, IMF, OECD, Eurostat, UN, and World Bank. (2020).
-#' *Consumer Price Index Manual: Theory and Practice*.
+#' IMF, ILO, Eurostat, UNECE, OECD, and World Bank. (2020).
+#' *Consumer Price Index Manual: Concepts and Methods*.
 #' International Monetary Fund.
 #'
 #' Lord, N. (2002). Does Smaller Spread Always Mean Larger Product?
@@ -445,7 +445,7 @@ extended_mean <- function(r, s) {
     } else if (r == s) {
       res <- exp((a^r * log(a) - b^r * log(b)) / (a^r - b^r) - 1 / r)
     } else {
-      res <- ((a^s - b^s) / (a^r - b^r) * r / s)^(1 / (s - r))
+      res <- ((a^s - b^s) / (a^r - b^r) * (r / s))^(1 / (s - r))
     }
     # Set output to a when a == b.
     i <- which(abs(a - b) <= tol)
@@ -606,6 +606,9 @@ lehmer_mean <- function(r) {
   function(x, w = NULL, na.rm = FALSE) {
     v <- x^(r - 1)
     if (!is.null(w)) {
+      if (length(x) != length(w)) {
+        stop("'x' and 'w' must be the same length")
+      }
       v <- v * w
     }
     arithmetic_mean(x, v, na.rm = na.rm)
@@ -653,7 +656,7 @@ contraharmonic_mean <- lehmer_mean(2)
 #' rather than removing all missing values prior to any calculations. This
 #' means that a different number of data points could be used to calculate the
 #' inner means. Use the [balanced()] operator to balance
-#' missing values across `w1` and w2 prior to any calculations.
+#' missing values across `w1` and `w2` prior to any calculations.
 #'
 #' @seealso
 #' [nested_contributions()] for percent-change contributions for
@@ -663,7 +666,7 @@ contraharmonic_mean <- lehmer_mean(2)
 #' Diewert, W. E. (1976). Exact and superlative index numbers.
 #' *Journal of Econometrics*, 4(2): 114--145.
 #'
-#' ILO, IMF, OECD, Eurostat, UN, and World Bank. (2004).
+#' ILO, IMF, OECD, UNECE, and World Bank. (2004).
 #' *Producer Price Index Manual: Theory and Practice*. International Monetary
 #' Fund.
 #'
