@@ -13,7 +13,7 @@ pythagorean_index <- function(r) {
       "Carli", "Dutot", "Laspeyres",
       "Palgrave", "Drobisch", "Unnamed",
       "Walsh1", "MarshallEdgeworth", "GearyKhamis",
-      "Lowe", "Young"
+      "Lowe", "Young", "HybridCSWD"
     )
   )
   gen_mean <- generalized_mean(r)
@@ -27,13 +27,16 @@ pythagorean_index <- function(r) {
       Dutot = ,
       Jevons = ,
       Coggeshall = function(p1, p0, na.rm = FALSE) {
+        check_pqs(p1, p0)
         gen_mean(p1 / p0, weights(p0), na.rm)
       },
       Laspeyres = function(p1, p0, q0, na.rm = FALSE) {
+        check_pqs(p1, p0, q0)
         gen_mean(p1 / p0, weights(p0, q0), na.rm)
       },
       Paasche = ,
       Palgrave = function(p1, p0, q1, na.rm = FALSE) {
+        check_pqs(p1, p0, q1)
         gen_mean(p1 / p0, weights(p1, q1), na.rm)
       },
       Drobisch = ,
@@ -44,22 +47,31 @@ pythagorean_index <- function(r) {
       Tornqvist = ,
       Theil = ,
       Rao = function(p1, p0, q1, q0, na.rm = FALSE) {
+        check_pqs(p1, p0, q1, q0)
         gen_mean(p1 / p0, weights(p1, p0, q1, q0), na.rm)
       },
       Vartia1 = ,
       MontgomeryVartia = function(p1, p0, q1, q0, na.rm = FALSE) {
+        check_pqs(p1, p0, q1, q0)
         exp(sum(log(p1 / p0) * weights(p1, p0, q1, q0), na.rm = na.rm))
       },
       Walsh1 = ,
       MarshallEdgeworth = ,
       GearyKhamis = function(p1, p0, q1, q0, na.rm = FALSE) {
+        check_pqs(p1, p0, q1, q0)
         gen_mean(p1 / p0, weights(p0, q1, q0), na.rm)
       },
       Lowe = function(p1, p0, qb, na.rm = FALSE) {
+        check_pqs(p1, p0, qb)
         gen_mean(p1 / p0, weights(p0, qb), na.rm)
       },
       Young = function(p1, p0, pb, qb, na.rm = FALSE) {
+        check_pqs(p1, p0, pb, qb)
         gen_mean(p1 / p0, weights(pb, qb), na.rm)
+      },
+      HybridCSWD = function(p1, p0, na.rm = FALSE) {
+        check_pqs(p1, p0)
+        gen_mean(p1 / p0, weights(p1, p0), na.rm)
       }
     )
   }
@@ -90,6 +102,7 @@ pythagorean_index <- function(r) {
 #' - Rao
 #' - Lowe
 #' - Young
+#' - Hybrid-CSWD
 #'
 #' The weights need not sum to 1, as this normalization isn't always
 #' appropriate (i.e., for the Vartia-I weights).
@@ -103,15 +116,29 @@ pythagorean_index <- function(r) {
 #'
 #' @note
 #' Naming for the indexes and weights generally follows the CPI manual (2020),
-#' Balk (2008), and Selvanathan and Rao (1994). In several cases two or more
-#' names correspond to the same weights (e.g., Paasche and Palgrave, or
-#' Sato-Vartia and Vartia-II). The calculations are given in the examples.
+#' Balk (2008), von der Lippe (2007), and Selvanathan and Rao (1994). In several
+#' cases two or more names correspond to the same weights (e.g., Paasche and
+#' Palgrave, or Sato-Vartia and Vartia-II). The calculations are given in the
+#' examples.
 #'
 #' @seealso
 #' [update_weights()] for price-updating weights.
 #'
 #' [quantity_index()] to remap the arguments in these functions for a
 #' quantity index.
+#' 
+#' @references
+#' Balk, B. M. (2008). *Price and Quantity Index Numbers*.
+#' Cambridge University Press.
+#'
+#' IMF, ILO, Eurostat, UNECE, OECD, and World Bank. (2020).
+#' *Consumer Price Index Manual: Concepts and Methods*.
+#' International Monetary Fund.
+#'
+#' von der Lippe, P. (2007). *Index Theory and Price Statistics*. Peter Lang.
+#'
+#' Selvanathan, E. A. and Rao, D. S. P. (1994).
+#' *Index Numbers: A Stochastic Approach*. MacMillan.
 #'
 #' @examples
 #' p0 <- price6[[2]]
@@ -227,6 +254,10 @@ pythagorean_index <- function(r) {
 #' # Young
 #'
 #' all.equal(index_weights("Young")(pb, qb), pb * qb)
+#' 
+#' # Hybrid CSWD (to approximate a CSWD index)
+#' 
+#' all.equal(index_weights("HybridCSWD")(p1, p0), sqrt(p0 / p1))
 #'
 #' @family price index functions
 #' @export
@@ -239,7 +270,7 @@ index_weights <- function(
       "Walsh1", "Walsh2", "MarshallEdgeworth",
       "GearyKhamis", "Vartia1", "MontgomeryVartia",
       "Vartia2", "SatoVartia", "Theil", "Rao",
-      "Lowe", "Young"
+      "Lowe", "Young", "HybridCSWD"
     )) {
   switch(match.arg(type),
     Carli = ,
@@ -249,50 +280,90 @@ index_weights <- function(
       p0
     },
     Dutot = function(p0) p0,
-    Young = function(pb, qb) pb * qb,
-    Lowe = function(p0, qb) p0 * qb,
+    Young = function(pb, qb) {
+      check_pqs(pb, qb)
+      pb * qb
+    },
+    Lowe = function(p0, qb) {
+      check_pqs(p0, qb)
+      p0 * qb
+    },
     LloydMoulton = ,
-    Laspeyres = function(p0, q0) p0 * q0,
-    HybridLaspeyres = function(p1, q0) p1 * q0,
+    Laspeyres = function(p0, q0) {
+      check_pqs(p0, q0)
+      p0 * q0
+    },
+    HybridLaspeyres = function(p1, q0) {
+      check_pqs(p1, q0)
+      p1 * q0
+    },
     Palgrave = ,
-    Paasche = function(p1, q1) p1 * q1,
-    HybridPaasche = function(p0, q1) p0 * q1,
+    Paasche = function(p1, q1) {
+      check_pqs(p1, q1)
+      p1 * q1
+    },
+    HybridPaasche = function(p0, q1) {
+      check_pqs(p0, q1)
+      p0 * q1
+    },
     Drobisch = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       v0 <- scale_weights(p0 * q0)
       v01 <- scale_weights(p0 * q1)
       (v0 + v01) / 2
     },
     Unnamed = ,
     Tornqvist = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       v0 <- scale_weights(p0 * q0)
       v1 <- scale_weights(p1 * q1)
       (v0 + v1) / 2
     },
-    Walsh1 = function(p0, q1, q0) p0 * sqrt(q0 * q1),
-    Walsh2 = function(p1, p0, q1, q0) sqrt(p0 * q0 * p1 * q1),
-    MarshallEdgeworth = function(p0, q1, q0) p0 * (q0 + q1),
-    GearyKhamis = function(p0, q1, q0) p0 / (1 / q0 + 1 / q1),
+    Walsh1 = function(p0, q1, q0) {
+      check_pqs(p0, q1, q0)
+      p0 * sqrt(q0 * q1)
+    },
+    Walsh2 = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
+      sqrt(p0 * q0 * p1 * q1)
+    },
+    MarshallEdgeworth = function(p0, q1, q0) {
+      check_pqs(p0, q1, q0)
+      p0 * (q0 + q1)
+    },
+    GearyKhamis = function(p0, q1, q0) {
+      check_pqs(p0, q1, q0)
+      p0 / (1 / q0 + 1 / q1)
+    },
     Vartia1 = ,
     MontgomeryVartia = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       v0 <- p0 * q0
       v1 <- p1 * q1
       logmean(v0, v1) / logmean(sum(v0, na.rm = TRUE), sum(v1, na.rm = TRUE))
     },
     Vartia2 = ,
     SatoVartia = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       v0 <- scale_weights(p0 * q0)
       v1 <- scale_weights(p1 * q1)
       logmean(v0, v1)
     },
     Theil = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       w0 <- scale_weights(p0 * q0)
       w1 <- scale_weights(p1 * q1)
       ((w0 + w1) / 2 * w0 * w1)^(1 / 3)
     },
     Rao = function(p1, p0, q1, q0) {
+      check_pqs(p1, p0, q1, q0)
       w0 <- scale_weights(p0 * q0)
       w1 <- scale_weights(p1 * q1)
       w0 * w1 / (w0 + w1)
+    },
+    HybridCSWD = function(p1, p0) {
+      check_pqs(p1, p0)
+      sqrt(p0 / p1)
     }
   )
 }
@@ -312,15 +383,16 @@ index_weights <- function(
 #' - Laspeyres
 #' - Palgrave
 #' - Unnamed index (arithmetic mean of Laspeyres and Palgrave)
-#' - Drobisch (arithmetic mean of Laspeyres and Paasche)
+#' - Drobisch (or Sidgwick, arithmetic mean of Laspeyres and Paasche)
 #' - Walsh-I (arithmetic Walsh)
 #' - Marshall-Edgeworth
 #' - Geary-Khamis
 #' - Lowe
 #' - Young
+#' - Hybrid-CSWD
 #' - **Geometric indexes**
 #' - Jevons
-#' - Geometric Laspeyres
+#' - Geometric Laspeyres (or Jöhr)
 #' - Geometric Paasche
 #' - Geometric Young
 #' - Törnqvist (or Törnqvist-Theil)
@@ -348,11 +420,12 @@ index_weights <- function(
 #' functions.
 #'
 #' In addition to these indexes, there are also functions for calculating a
-#' variety of indexes not based on generalized means. The Fisher index is the
+#' variety of indexes based on nested generalized means. The Fisher index is the
 #' geometric mean of the arithmetic Laspeyres and Paasche indexes; the Harmonic
-#' Laspeyres Paasche index is the harmonic analog of the Fisher index (8054 on
-#' Fisher's list). The Carruthers-Sellwood-Ward-Dalen and
-#' Carruthers-Sellwood-Ward-Dalen-Balk indexes are sample analogs of the Fisher
+#' Laspeyres Paasche (or Harmonic Paasche Laspeyres) index is the harmonic
+#' analog of the Fisher index (8054 on Fisher's list). The
+#' Carruthers-Sellwood-Ward-Dalen and Carruthers-Sellwood-Ward-Dalen-Balk
+#' indexes are sample analogs of the Fisher
 #' index; the Balk-Walsh index is the sample analog of the Walsh index. The AG
 #' mean index is the arithmetic or geometric mean of the geometric and
 #' arithmetic Laspeyres indexes, weighted by the elasticity of substitution.
@@ -421,13 +494,11 @@ index_weights <- function(
 #' Fisher, I. (1922). *The Making of Index Numbers*. Houghton Mifflin
 #' Company.
 #'
-#' ILO, IMF, OECD, Eurostat, UN, and World Bank. (2020).
-#' *Consumer Price Index Manual: Theory and Practice*.
+#' IMF, ILO, Eurostat, UNECE, OECD, and World Bank. (2020).
+#' *Consumer Price Index Manual: Concepts and Methods*.
 #' International Monetary Fund.
 #'
-#' von der Lippe, P. (2001).
-#' *Chain Indices: A Study in Price Index Theory*, Spectrum of Federal
-#' Statistics vol. 16. Federal Statistical Office, Wiesbaden.
+#' von der Lippe, P. (2007). *Index Theory and Price Statistics*. Peter Lang.
 #'
 #' von der Lippe, P. (2015). Generalized Statistical Means and New Price Index
 #' Formulas, Notes on some unexplored index formulas, their interpretations and
@@ -465,17 +536,15 @@ index_weights <- function(
 #'
 #' all.equal(
 #'   scale_weights(index_weights("HybridLaspeyres")(p1, q0)),
-#'   scale_weights(
-#'     transmute_weights(1, -1)(p1 / p0, index_weights("Laspeyres")(p0, q0))
-#'   )
+#'   transmute_weights(1, -1)(p1 / p0, index_weights("Laspeyres")(p0, q0))
 #' )
 #'
 #' # This strategy can be used to make more exotic indexes, like the
-#' # quadratic-mean index (von der Lippe, 2001, p. 71)
+#' # quadratic-mean index (von der Lippe, 2007, p. 61)
 #'
 #' generalized_mean(2)(p1 / p0, index_weights("Laspeyres")(p0, q0))
 #'
-#' # Or the exponential mean index (p. 64)
+#' # Or the exponential mean index (p. 62)
 #'
 #' log(arithmetic_mean(exp(p1 / p0), index_weights("Laspeyres")(p0, q0)))
 #'
@@ -644,6 +713,7 @@ nested_index <- function(r, s) {
   nest_mean <- nested_mean(r, s)
 
   function(p1, p0, q1, q0, na.rm = FALSE) {
+    check_pqs(p1, p0, q1, q0)
     nest_mean(p1 / p0, p0 * q0, p1 * q1, na.rm)
   }
 }
@@ -665,6 +735,7 @@ lm_index <- function(elasticity) {
   gen_mean <- generalized_mean(1 - elasticity)
 
   function(p1, p0, q0, na.rm = FALSE) {
+    check_pqs(p1, p0, q0)
     gen_mean(p1 / p0, p0 * q0, na.rm)
   }
 }
@@ -673,6 +744,7 @@ lm_index <- function(elasticity) {
 #' @rdname price_indexes
 #' @export
 cswd_index <- function(p1, p0, na.rm = FALSE) {
+  check_pqs(p1, p0)
   fisher_mean(p1 / p0, na.rm = na.rm)
 }
 
@@ -680,6 +752,7 @@ cswd_index <- function(p1, p0, na.rm = FALSE) {
 #' @rdname price_indexes
 #' @export
 cswdb_index <- function(p1, p0, q1, q0, na.rm = FALSE) {
+  check_pqs(p1, p0, q1, q0)
   sqrt(arithmetic_mean(p1 / p0, na.rm = na.rm) /
     arithmetic_mean(q1 / q0, na.rm = na.rm) *
     arithmetic_mean(p1 * q1 / (p0 * q0), na.rm = na.rm))
@@ -689,6 +762,7 @@ cswdb_index <- function(p1, p0, q1, q0, na.rm = FALSE) {
 #' @rdname price_indexes
 #' @export
 bw_index <- function(p1, p0, na.rm = FALSE) {
+  check_pqs(p1, p0)
   rel <- sqrt(p1 / p0)
   arithmetic_mean(rel, na.rm = na.rm) * harmonic_mean(rel, na.rm = na.rm)
 }
@@ -707,6 +781,7 @@ stuvel_index <- function(a, b) {
   }
 
   function(p1, p0, q1, q0, na.rm = FALSE) {
+    check_pqs(p1, p0, q1, q0)
     v0 <- p0 * q0
     v1 <- p1 * q1
     pl <- arithmetic_mean(p1 / p0, v0, na.rm)
@@ -724,6 +799,7 @@ agmean_index <- function(r) {
   function(elasticity) {
     nest_mean <- nested_mean(r, c(0, 1), c(elasticity, 1 - elasticity))
     function(p1, p0, q0, na.rm = FALSE) {
+      check_pqs(p1, p0, q0)
       v0 <- p0 * q0
       nest_mean(p1 / p0, v0, v0, na.rm)
     }
@@ -744,6 +820,7 @@ geometric_agmean_index <- agmean_index(0)
 #' @rdname price_indexes
 #' @export
 lehr_index <- function(p1, p0, q1, q0, na.rm = FALSE) {
+  check_pqs(p1, p0, q1, q0)
   v1 <- p1 * q1
   v0 <- p0 * q0
   v <- (v1 + v0) / (q1 + q0)
